#ifndef _CONFIG_H_
#   define _CONFIG_H_

#include <string>
#include <vector>

using std::string;
using std::vector;

// -- Section Names

#define CONFIG_OPTIONS "options"
#define CONFIG_ENTRY   "entry"

// -- Key Names

#define CONFIG_TO           "to"
#define CONFIG_UPDATE       "update"
#define CONFIG_VERBOSE      "verbose"
#define CONFIG_FORCE        "force"
#define CONFIG_LOG          "log"
#define CONFIG_CONSOLE      "console"
#define CONFIG_SHOW_ERROR   "show_error"
#define CONFIG_SHOW_SUCCESS "show_success"

#define CONFIG_FROM    "from"
#define CONFIG_RECURSE "recurse"
#define CONFIG_HIDDEN  "hidden"
#define CONFIG_SYSTEM  "system"
#define CONFIG_EXCLUDE "exclude"

/*****************************************************************************
 * Declaration: class Config
 *****************************************************************************/

struct Config
{
   // ---------------------------------
   // Our 'entry' settings
   // ---------------------------------

   struct Entry
   {
      string mTo;
      string mUpdate;
      string mFrom;
      string mRecurse;
      string mHidden;
      string mSystem;

      vector<string> mExcludeList;
   };

   // ---------------------------------
   // Our 'options' settings
   // ---------------------------------

   struct Options
   {
      string mTo;
      string mUpdate;
      string mHidden;
      string mSystem;
      string mVerbose;
      string mForce;
      string mLog;
      string mConsole;
      string mShowError;
      string mShowSuccess;
   };

   // Member Variables

   Options       mOptions;
   vector<Entry> mEntries;
   Entry         mDefaultEntry;
   Entry         *mCurrentEntry;

   // ---------------------------------
   // Constructor
   // ---------------------------------

   Config();

   // ---------------------------------
   // Utility functions
   // ---------------------------------

   static string entryToString(Entry &theEntry, string title);
   static void dumpEntry(Entry &theEntry, string title);

   string toString(string title, int showDefaultEntry);
   void dump(string title);

   static string kvToString(string key, string value);

   static void debug(string msg);
   static void debug(string section, string key, string value);

   // ---------------------------------------------------------
   // Called when starting to process an 'options' section
   // ---------------------------------------------------------

   static int optionsBegin(
      void *voidPtr, string key, string value, string &errMsg
   );

   // ---------------------------------------------------------
   // Option value handler functions
   // ---------------------------------------------------------

   static int optionsTo(
      void *voidPtr, string key, string value, string &errMsg
   );

   static int optionsUpdate(
      void *voidPtr, string key, string value, string &errMsg
   );

   static int optionsHidden(
      void *voidPtr, string key, string value, string &errMsg
   );

   static int optionsSystem(
      void *voidPtr, string key, string value, string &errMsg
   );

   static int optionsVerbose(
      void *voidPtr, string key, string value, string &errMsg
   );

   static int optionsForce(
      void *voidPtr, string key, string value, string &errMsg
   );

   static int optionsLog(
      void *voidPtr, string key, string value, string &errMsg
   );

   static int optionsConsole(
      void *voidPtr, string key, string value, string &errMsg
   );

   static int optionsShowError(
      void *voidPtr, string key, string value, string &errMsg
   );

   static int optionsShowSuccess(
      void *voidPtr, string key, string value, string &errMsg
   );

   // ---------------------------------------------------------
   // Called when starting to process an 'entry' section
   // ---------------------------------------------------------

   static int entryBegin(
      void *voidPtr, string key, string value, string &errMsg
   );

   // ---------------------------------------------------------
   // Entry value handler functions
   // ---------------------------------------------------------

   static int entryTo(
      void *voidPtr, string key, string value, string &errMsg
   );

   static int entryUpdate(
      void *voidPtr, string key, string value, string &errMsg
   );

   static int entryFrom(
      void *voidPtr, string key, string value, string &errMsg
   );

   static int entryRecurse(
      void *voidPtr, string key, string value, string &errMsg
   );

   static int entryHidden(
      void *voidPtr, string key, string value, string &errMsg
   );

   static int entrySystem(
      void *voidPtr, string key, string value, string &errMsg
   );

   static int entryExclude(
      void *voidPtr, string key, string value, string &errMsg
   );
};

#endif
