/*****************************************************************************
 * C/C++ Includes
 *****************************************************************************/

#include <stdio.h>
#include <time.h>

/*****************************************************************************
 * Local Includes
 *****************************************************************************/

#include "messagehandler.h"
#include "util.h"
#include "dir.h"

MessageHandler *MessageHandler::get(
   int doShowError, int doShowSuccess, string &errMsg
)
{
   time_t now;
   time(&now);

   // ----------------------------------
   // Attempt to clean out old files
   // ----------------------------------

   time_t cutoff = now - DAYS_CUTOFF * 24 * 60 * 60;

   DirEntryVector v;

   if(!Dir::listRegex("\\.html$", ".", v, errMsg))
      return(0);

   for(int n = v.size(), i = 0; i < n; i ++)
   {
      DirEntry theEntry = v.at(i);

      if(theEntry.mModTime < cutoff)
      {
         const char *name = theEntry.mFullPath.c_str();

         if(0)
         {
            printf("deleting: [%s]\n", name);
         }

         if(unlink(name) != 0)
         {
            errMsg = "Could not delete: [" + theEntry.mFullPath + "]";
            return(0);
         }
      }
   }

   // Work out our file name, should
   // look like:
   //    2006_Jan_06.html

   const char *s    = ctime(&now);
   char buffer[255] = { '\0' };

   sprintf(
      buffer,
      "%.4s_%.3s_%.2s.html",
      s + 20,
      s + 4,
      s + 8
   );

   // Now, attempt to open the file and write 4K of white space
   // to it as a way of reserving space in case we run
   // out of disk space

   FILE *f = fopen(buffer, "w");

   if(!f)
   {
      errMsg  = "Could not open file: [";
      errMsg += buffer;
      errMsg += "]";

      return(0);
   }

   for(int j = 0; j < 4096; j ++)
      fprintf(f, " ");

   fclose(f);

   return(
      new MessageHandler(
         doShowError, doShowSuccess, buffer
      )
   );
}

static void doExit(
   ForceAlertList &fList, string fileName, string msg, int show
)
{
   const char *ptr = fileName.c_str();
   FILE *f         = fopen(ptr, "w");

   if(!f)
   {
      printf("Error: Could not open file: [%s]\n", ptr);
      exit(-1);
   }

   time_t now;

   time(&now);

   string theTime = ctime(&now);
   string fString = "";
   int n          = fList.size();

   if(n > 0)
   {
      for(int i = 0; i < n; i ++)
      {
         fString += fList[i] + "<br>\n";
      }

      fString = "<hr>\n" + fString;
   }

   fprintf(
      f,
      "<html>\n"
      " <head><title>%s</title></head>\n"
      " <body>\n"
      "  <center>\n"
      "   <h2>Backup Results For: %s</h2>\n"
      "  </center>\n"
      "  %s\n"
      "  <hr>\n"
      "  <p>\n"
      "  %s\n"
      " </body>\n"
      "</html>\n",
      fileName.c_str(),
      Util::trim(theTime).c_str(),
      fString.c_str(),
      msg.c_str()
   );

   fclose(f);

   if(show)
   {
      string cmd = "start " + fileName;
      system(cmd.c_str());
   }
}

void MessageHandler::exitSuccess(ForceAlertList &fList, string msg)
{
   doExit(fList, mFileName, msg, mDoShowSuccess);
   exit(0);
}

void MessageHandler::exitError(ForceAlertList &fList, string msg)
{
   doExit(fList, mFileName, msg, mDoShowError);
   exit(-1);
}

MessageHandler::MessageHandler(
   int doShowError, int doShowSuccess, string fileName
) :
   mDoShowError(doShowError),
   mDoShowSuccess(doShowSuccess),
   mFileName(fileName)
{
}

#ifdef __MAIN__

int main()
{
   MessageHandler handler(1, 1);

   string errMsg;

   if(!handler.doBegin(errMsg))
   {
      printf("[ERROR] {%s}\n", errMsg.c_str());
   }
   else
   {
      printf("Ok\n");
   }

   handler.exitSuccess("Dig it ...");

   return(0);
}
#endif
