/*****************************************************************************
 * C/C++ Headers
 *****************************************************************************/

#include <stdio.h>
#include <iostream>

using std::cout;
using std::endl;

/*****************************************************************************
 * Local Headers
 *****************************************************************************/

#include "output.h"

/*****************************************************************************
 * class: OutputToConsole
 *****************************************************************************/

OutputToConsole::OutputToConsole(int verbose) :
   mVerbose(verbose)
{
}

void OutputToConsole::write(string s)
{
   cout << s << endl;
}

void OutputToConsole::writeVerbose(string s)
{
   if(mVerbose)
      write(s);
}

/*****************************************************************************
 * class: OutputToFile
 *****************************************************************************/

OutputToFile::OutputToFile(int verbose, string fileName) :
   mVerbose(verbose),
   mFileName(fileName)
{
}

void OutputToFile::write(string s)
{
   FILE *f          = NULL;
   const char *name = mFileName.c_str();

   if((f = fopen(name, "a")) == NULL)
   {
      printf("[Error] - Could not open file: [%s]\n", name);
   }
   else
   {
      fprintf(f, "%s\n", s.c_str());
      fclose(f);
   }
}

void OutputToFile::writeVerbose(string s)
{
   if(mVerbose)
      write(s);
}

/*****************************************************************************
 * class: OutputTee
 *****************************************************************************/

OutputTee::OutputTee(Output *first, Output *second) :
   mFirst(first), mSecond(second)
{
}

void OutputTee::write(string s)
{
   mFirst->write(s);
   mSecond->write(s);
}

void OutputTee::writeVerbose(string s)
{
   mFirst->writeVerbose(s);
   mSecond->writeVerbose(s);
}

OutputTee::~OutputTee()
{
   delete mFirst;
   delete mSecond;
}

#ifdef __MAIN__

int main(int argc, char **argv)
{
   OutputToConsole oc(1);
   OutputToFile    of(1);

   string errMsg;

   if(!of.begin("blah.txt", errMsg))
   {
      cout << errMsg << endl;
   }
   else
   {
      OutputTee t(oc, of);

      t.write("Hello there, sailor!");
      t.writeVerbose("BooGah BooGah BooGah");
   }

   return(0);
}

#endif
