#include <stdio.h>

#include "regex.h"
#include "pcre.h"

int Regex::match(
   const char *pattern,
   int ignoreCase,
   const char *target,
   EntryVector &instances,
   string &errMsg
)
{
   int errorOffset      = 0;
   const char *errorPtr = 0;
   pcre *re             = NULL;

   if(0)
      printf("pattern: [%s]\n", pattern);

   errMsg = "";

   re = pcre_compile(
      pattern,
      ignoreCase ? PCRE_CASELESS : 0,
      &errorPtr,
      &errorOffset,
      NULL
   );

   if(!re)
   {
      errMsg += "Error: ";
      errMsg += errorPtr;
      errMsg += ", ";
      errMsg += pattern;

      if(pattern + errorOffset < pattern + strlen(pattern))
      {
         // We've NOT been given an errorOffset that lands
         // us at the end of the 'pattern'.
         //
         // Now, we're either at the beginning of
         // the string, or, we'll supply offset information

         if(errorOffset > 0)
         {
            char tmp[255] = { '\0' };

            sprintf(tmp, "%d", errorOffset);

            errMsg += ", at offset: ";
            errMsg += tmp;
            errMsg += ", ";
            errMsg += pattern + errorOffset;
         }
      }

      return(0);
   }

   // The pattern was ok.  From here on out,
   // we're obilgated to free(re) before
   // we return.

   int ovector[255] = { 0 };
   int size         = sizeof(ovector) / sizeof(ovector[0]);
   int nAdded       = 0;
   int theOffset    = 0;
   int returnValue  = 1;

   while(1)
   {
      const char *segment = target + theOffset;

      if(0)
         printf("searching segment: '%s'\n", segment);

      if(!*segment)
         break;

      int rv = pcre_exec(
         re,
         NULL,
         segment,
         strlen(segment),
         0,
         0,
         ovector,
         size
      );

      if(0)
         printf("returnValue: [%d]\n", rv);

      if(rv <= 0)
      {
         returnValue = 0;

         // The only "error" that doesn't cause us
         // to populate errMsg is a non-match

         if(rv != PCRE_ERROR_NOMATCH)
         {
            char buffer[255] = { '\0' };

            sprintf(buffer, "Error: returnCode [%d]", rv);

            errMsg = buffer;
         }
         else
         {
            // A non match, if this is our first
            // pass through the loop (no previous matches)
            // then we return 0 indicating that a match
            // was not found.

            if(theOffset > 0)
               returnValue = 1;
         }

         break;
      }

      // Record the matches (strings/offsets)

      Entry theEntry;

      for(int i = 0, j = 0; i < rv; i++, j += 2)
      {
         int start = ovector[j];
         int end   = ovector[j + 1];

         // Work around a bit of weirdness we saw
         // when using: "^\\s*"

         if(start == 0 && end == 0)
         {
            if(rv == 1)
            {
               end = strlen(segment);
            }
         }

         if(0)
         {
            printf("j: %d, start: %d, end: %d\n", j, start, end);
         }

         int n = (end - start);

         char *p = new char[n + 1];

         sprintf(p, "%.*s", n, segment + start);

         string theString = p;

         theEntry.mStrings.push_back(theString);

         delete[] p;

         theEntry.mOffsets.push_back(theOffset + start);

         if(i + 1 == rv)
         {
            theOffset += end;
         }
      }

      instances.push_back(theEntry);
   }

   free(re);

   return(returnValue);
}

#ifdef __MAIN__

int main()
{
   //char *pattern  = "([0-9]{3})-([0-9]{3})-([0-9]{4})";
   //char *pattern  = "^\\s*[^\\s]+";
   //char *pattern  = "dog[^\\s]*";
   char *pattern  = "(dog[^\\s]*)\\s*(cat[^\\s]*)";
   int ignoreCase = 1;
   
   //const char *target = "------ -------- -------- 0000 882-393-3333";
   //const char *target = "      	 	 	.";

   const char *target =
      "ziggy ziggy Dog1 cat dogXXX cat dog__ cat doG..... cat";

   int z = 0;

   while(z++ < 1)
   {
      EntryVector ev;

      string errMsg;

      int status = Regex::match(
         pattern,
         ignoreCase,
         target,
         ev,
         errMsg
      );

      if(!status)
      {
         if(errMsg == "")
         {
            printf("No Match\n");
         }
         else
         {
            printf("%s\n", errMsg.c_str());
         }
      }
      else
      {
         printf("Regex : [%s]\n", pattern);
         printf("Target: [%s]\n", target);

         int i = ev.size();

         printf("size: [%d]\n", i);

         for(int j = 0; j < i; j ++)
         {
            Entry e = ev.at(j);
            int n   = e.mStrings.size();

            printf("Entry: [%d]\n", j);

            for(int k = 0; k < n; k ++)
            {
               string s   = e.mStrings.at(k);
               int offset = e.mOffsets.at(k);

               printf(
                  "[%d][%s][%d]\n", k, s.c_str(), offset
               );
            }
         }
      }
   }

   return(0);
}

#endif
