/*****************************************************************************
 * C/C++ Headers
 *****************************************************************************/

#include <stdio.h>
#include <set>
#include <string>
#include <time.h>

using std::set;
using std::string;

/*****************************************************************************
 * LocalHeaders
 *****************************************************************************/

#include "util.h"
#include "stringset.h"

typedef set<string> SSet;

/*****************************************************************************
 * Definition: struct StringSetP
 *****************************************************************************/

struct IStringSet
{
   IStringSet(int ignoreCase);
   string getString(string s);

   int mIgnoreCase;
   SSet mSet;
};

IStringSet::IStringSet(int ignoreCase) :
   mIgnoreCase(ignoreCase)
{
}

string IStringSet::getString(string s)
{
   if(mIgnoreCase)
      return(Util::toLowerCase(s));

   return(s);
}

/*****************************************************************************
 * Definition: class StringSet
 *****************************************************************************/

StringSet::StringSet(int ignoreCase) :
   mInternal(new IStringSet(ignoreCase))
{
}

StringSet::~StringSet()
{
   delete mInternal;
}

vector<string> StringSet::getList()
{
   vector<string> theList;

   SSet::iterator current = mInternal->mSet.begin();
   SSet::iterator end     = mInternal->mSet.end();

   while(current != end)
   {
      theList.push_back(*current);
      current++;
   }

   return(theList);
}

int StringSet::count()
{
   return(mInternal->mSet.size());
}

void StringSet::insert(string s)
{
   mInternal->mSet.insert(mInternal->getString(s));
}

void StringSet::remove(string s)
{
   mInternal->mSet.erase(mInternal->getString(s));
}

void StringSet::clear()
{
   mInternal->mSet.clear();
}

int StringSet::contains(string s)
{
   if(mInternal->mSet.size() == 0)
      return(0);

   string theString = mInternal->getString(s);

   return(
      mInternal->mSet.find(theString) != mInternal->mSet.end()
   );
}

void StringSet::dump(FILE *out)
{
   SSet::iterator current = mInternal->mSet.begin();
   SSet::iterator end     = mInternal->mSet.end();

   while(current != end)
   {
      fprintf(out, "[%s]\n", current->c_str());

      current++;
   }
}

#ifdef __MAIN__

static void load(StringSet &ss)
{
   string list[] =
   {
      "g:\\foo\\bar",
      "z:\\",
      "z:",
      "a:\\foo\\",
      "b:\\blah\\",
      "b:\\x\\blah\\",
      "b:\\x\\y\\blah\\",
      "v:\\v\\v\\v\\v\\v\\v\\v\\",
      "v:\\v\\v\\v\\v\\v\\v\\v",

      // Repeats ...

      "g:\\foo\\bar",
      "z:",
      "z:",
      "z:",
      "b:\\x\\y\\blah\\",
      "b:\\x\\y\\blah\\",
      "v:\\v\\v\\v\\v\\v\\v\\v\\",
      "v:\\v\\v\\v\\v\\v\\v\\v"
   };

   int n = USZ(list);

   for(int i = 0; i < n; i ++)
      ss.insert(list[i]);
}

int main()
{
   StringSet ss(1);

   load(ss);

   printf("--- [Loaded] ---\n");

   ss.dump(stdout);

   ss.clear();

   printf("--- [Cleared] ---\n");

   ss.dump(stdout);

   load(ss);

   printf("--- [Re-Loaded] ---\n");

   ss.dump(stdout);

   ss.remove("Z:");
   ss.remove("v:\\v\\v\\v\\v\\v\\v\\v\\");
   ss.remove("b:\\x\\y\\blah\\");
   ss.remove("xxx:\\yyy");

   printf("--- [Removals] ---\n");

   ss.dump(stdout);

   printf("--- [Membership] ---\n");

   string list[] = {
      "g:\\foo\\bar",
      "Z:",
      "a:\\foo\\",
      "b:\\blah\\blah\\blah\\blah",
      "b:",
      "b:\\",
      "b:\\booger"
   };

   time_t start, end;

   int n = USZ(list);

   time(&start);

   for(int runs = 0; runs < 100000; runs ++)
   {
      for(int i = 0; i < n; i ++)
      {
         string seek = list[i];

         printf(
            "%-25s => %sfound\n",
            seek.c_str(),
            !ss.contains(list[i]) ?
               "NOT " : ""
         );
      }
   }

   time(&end);

   printf("elapsed: %d\n", end - start);

   return(0);
}

#endif
