/*****************************************************************************
 * C/C++ Headers
 *****************************************************************************/

#include <stdio.h>
#include <ctype.h>
#include <time.h>

/*****************************************************************************
 * Local Headers
 *****************************************************************************/

#include "util.h"

void Util::bail(string errMsg)
{
   printf("Error: [%s]\n", errMsg.c_str());
   exit(-1);
}

string Util::getTimeStamp()
{
   time_t now;

   time(&now);

   string s = ctime(&now);

   return(trim(s));
}

string Util::toLowerCase(string s)
{
   string returnValue = "";
   const char *ptr    = s.c_str();

   while(*ptr)
   {
      char c = *ptr;

      if(!islower(*ptr))
         c = tolower(*ptr);

      returnValue += c;
      ptr++;
   }

   return(returnValue);
}

string Util::ltrim(string s)
{
   string returnValue = "";
   const char *ptr    = s.c_str();

   while(*ptr && isspace(*ptr))
      ptr++;

   if(*ptr)
      returnValue = ptr;

   return(returnValue);
}

string Util::rtrim(string s)
{
   string returnValue = "";
   const char *ptr    = s.c_str();
   int length         = strlen(ptr);

   if(length > 0)
   {
      int index = -1;

      for(int i = 0; i < length; i ++)
      {
         char c = ptr[i];

         if(!isspace(c))
            index = i;
      }

      // If found, index will be the index of the last
      // non-whitespace char

      if(index >= 0)
      {
         // Copy up to and including 'index' ...

         for(int j = 0; j <= index; j ++)
            returnValue += ptr[j];
      }
   }

   return(returnValue);
}

string Util::trim(string s)
{
   return(Util::ltrim(Util::rtrim(s)));
}

string Util::lpad(string value, int length)
{
   int n = value.length();

   while(n < length)
   {
      value = " " + value;
      n++;
   }

   return(value);
}

string Util::rpad(string value, int length)
{
   int n = value.length();

   while(n < length)
   {
      value = value + " ";
      n++;
   }

   return(value);
}

string Util::substring(string s, int offset, int nChars)
{
   string returnValue = "";
   const char *ptr    = s.c_str();
   int length         = strlen(ptr);

   if(offset >= 0 && offset < length)
   {
      for(int i = 0; i < nChars; i ++)
      {
         int theIndex = offset + i;
         char c       = ptr[theIndex];

         if(!c)
            break;

         returnValue += c;
      }
   }

   return(returnValue);
}

string Util::substring(string s, int offset)
{
   return(Util::substring(s, offset, s.length() - offset));
}


vector<string> Util::split(string s, char ch)
{
   int n          = s.length();
   int inString   = 0;
   string current = "";

   vector<string> list;

   for(int i = 0; i < n; i ++)
   {
      char c = s.at(i);

      if(!inString)
      {
         if(c != ch)
         {
            inString = 1;
            current += c;
         }
      }
      else
      {
         int doAdd = 0;

         if(c != ch)
         {
            current += c;
         }
         else
         {
            list.push_back(current);

            current  = "";
            inString = 0;
         }
      }
   }

   if(current != "")
      list.push_back(current);

   return(list);
}

string Util::toString(const char *fmt, int value)
{
   char buffer[255] = { '\0' };

   sprintf(buffer, fmt, value);

   return(buffer);
}

string Util::toString(int value)
{
   return(Util::toString("%d", value));
}

string Util::toString(__int64 value)
{
   char buffer[255]   = { '\0' };

   sprintf(buffer, "%I64u", value);

   return(buffer);
}

string Util::toString(const char *fmt, double value)
{
   char buffer[255]   = { '\0' };

   sprintf(buffer, fmt, value);

   return(buffer);
}

string Util::toString(double value)
{
   return(Util::toString("%f", value));
}

string Util::empty()
{
   string s = "";

   return(s);
}


int Util::startsWith(string theString, string lookFor)
{
   const char *p1 = theString.c_str();
   const char *p2 = lookFor.c_str();

   int n = lookFor.length();

   for(int i = 0; i < n; i ++)
   {
      if(!p2[i] || p2[i] != p1[i])
         return(0);
   }

   return(1);
}

int Util::endsWith(string theString, string lookFor)
{
   int n1 = theString.length();
   int n2 = lookFor.length();

   if(n1 == 0 || n2 == 0 || n2 > n1)
      return(0);

   string s1 = Util::substring(theString, n1 - n2);

   return(s1 == lookFor);
}

int Util::indexOf(string theString, char ch)
{
   const char *p = theString.c_str();
   int i         = 0;

   while(p[i])
   {
      if(ch == p[i])
         return(i);

      i++;
   }

   return(-1);
}

int Util::lastIndexOf(string theString, char ch)
{
   int n = theString.length();

   if(n > 0)
   {
      for(int i = n - 1; i >= 0; i --)
         if(theString.at(i) == ch)
            return(i);
   }

   return(-1);
}

int Util::count(string theString, char ch)
{
   const char *p = theString.c_str();
   int n         = 0;

   while(*p)
   {
      if(ch == *p)
        n++;

      p++;
   }

   return(n);
}

int Util::imatch(string s1, string s2)
{
   s1 = Util::toLowerCase(s1);
   s2 = Util::toLowerCase(s2);

   return(s1 == s2);
}

int Util::isEmpty(string s)
{
   return(Util::trim(s).length() == 0);
}

#ifdef __MAIN__

int main()
{
   vector<string> v = Util::split("x|||yy||||z|z|a|b|c", '|');
   int size         = v.size();

   printf("Size: [%d]\n", size);

   for(int i = 0; i < size; i ++)
      printf("[%s]\n", v[i].c_str());

   return(0);
}

#endif
